@@ -3,8 +3,12 @@
 import os
 import re
 import subprocess
+import time
+
 import thornpy
-from .constants import TO_PARAMETER_PATTERN, TO_LENGTH_PARAM, ADRILL_IDS, ADRILL_PLUGIN_VAR
+import numpy as np
+
+from .constants import TO_PARAMETER_PATTERN, TO_LENGTH_PARAM, ADRILL_IDS, ADRILL_PLUGIN_VAR, ACF_FUNNEL_PATTERN, ACF_INTEGRATOR_ERROR_PATTERN
 from . import TMPLT_ENV
 
 def turn_measure_on(string_file, tool_types=[], tool_numbers=[], tool_names=[]):
@@ -842,12 +846,12 @@ def build(string_file, solver_settings_file, working_directory, output_name=None
     
     # Set the event filename and solver settings file name (relative paths if in the same directory)    
     event_file = thornpy.utilities.convert_path(get_TO_param(string_file, 'Event_Property_File'))
-    if os.path.split(event_file)[0] == thornpy.utilities.convert_path(working_directory):
+    if os.path.dirname(event_file) == thornpy.utilities.convert_path(working_directory):
         evt_name = os.path.split(event_file)[-1]
     else:
         evt_name = thornpy.utilities.convert_path(event_file)
     
-    if os.path.split(solver_settings_file)[0] == thornpy.utilities.convert_path(working_directory):
+    if os.path.dirname(solver_settings_file) == thornpy.utilities.convert_path(working_directory):
         ssf_name = os.path.split(solver_settings_file)[-1]
     else:
         ssf_name = os.path.split(solver_settings_file)
@@ -860,6 +864,12 @@ def build(string_file, solver_settings_file, working_directory, output_name=None
     cmds.append(f'file adams write file="{adm_file}"\n')
     cmds.append(f'simulation script write_acf sim_script_name = "{output_name}" file_name = "{acf_file}"\n')
     cmds.append(f'file command write entity_name = "{output_name}" file_name = "{cmd_file}"')
+
+    # If the working directory doesn't exit, create it
+    if not os.path.exists(working_directory):
+        os.makedirs(working_directory)
+
+    # Create the build.cmd script
     with open(os.path.join(working_directory, 'build.cmd'), 'w') as fid:			
         for cmd in cmds:
             fid.write(cmd)
@@ -868,13 +878,27 @@ def build(string_file, solver_settings_file, working_directory, output_name=None
     startupinfo = subprocess.STARTUPINFO()
     startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
     process = subprocess.Popen('"{}" aview ru-s b build.cmd'.format(os.environ['ADAMS_LAUNCH_COMMAND']), cwd=working_directory, startupinfo=startupinfo)
-    if wait:
-        process.wait()    
 
     adm_file = os.path.join(working_directory, adm_file)
     acf_file = os.path.join(working_directory, acf_file)
     cmd_file = os.path.join(working_directory, cmd_file)
 
+    # Wait for the process to finish (if requested)
+    if wait:        
+        while True:
+            if process.poll() is not None:
+                # If the process has terminated, stop waiting
+                break
+
+            elif os.path.exists(adm_file) and os.path.exists(acf_file) and os.path.exists(cmd_file):
+                # If the process hasn't terminated, but the adm file acf file and cmd files all exist
+                time.sleep(5)
+                break
+            
+            else:
+                # If the model is still being built
+                time.sleep(1)
+                
     return adm_file, acf_file, cmd_file
 
 TO_BLOCK_HEADER_PATTERN = re.compile('^\\[[_0-9a-zA-Z]+\\]\\s*$') 
@@ -1171,6 +1195,332 @@ def add_splines_to_acf(acf_file):
     os.remove(acf_file)
     os.rename(acf_file.replace('.acf','.tmp'), acf_file)
 
+def modify_acf_solver_settings(acf_file, statics=None, error=None):
+    """Modifies the contents of the Adams Command (.acf) file given in `acf_file` to apply the static funnel given in `statics` and the dynamic steps given in `dynamics`.
+    
+    Parameters
+    ----------
+    acf_file : str
+        Path to Adams Command (.acf) file to be updated
+    statics : list
+        List of six lists where each list contains the values of a particular equilibrium setting at each step in the funnel. (Defaults to the value already in the acf file.)
+    error : float
+        Error tolerance to use for the dynamics solver. (Defaults to the value already in the acf file.)
+
+    Example
+    -------    
+    The following example modifies example.acf to have a three-step static funnel and a dynamic solver error tolerance of 0.0001.
+
+    >>> maxit = [500, 500, 500]
+    >>> stab = [8.0, 1.0, .01]
+    >>> error = [1.0, 0.1, .01]
+    >>> imbal = [4.0, 2.0, .01]
+    >>> tlim = [4.0, 2.0, 1.0]
+    >>> alim = [4.0, 2.0, 1.0] # Assumes DEGREES
+    >>> statics = [maxit, stab, error, imbal, tlim, alim]
+    >>> modify_acf_solver_settings('example.acf', statics=statics, error=.0001)
+
+    """
+    # Read the current acf text
+    with open(acf_file, 'r') as fid:
+        acf_text = fid.read()
+    
+    # Replace the static funnel
+    if statics is not None:
+        if len(statics) != 6 or not all([len(statics[0])==len(param) for param in statics]):
+            raise Exception('The statics list does not have the correct shape.')
+
+        # Create the new funnel text
+        funnel = ''.join([f'equil/maxit={maxit}, stab={stab}, error={error}, imbal={imbal}, tlim={tlim}, alim={alim}D\nsim/stat\n' for maxit, stab, error, imbal, tlim, alim in zip(*statics)])
+        
+        # Replace the funnel
+        acf_text = ACF_FUNNEL_PATTERN.sub('!\n!\n' + funnel, acf_text)
+    
+    # Replace the dynamic error tolerance
+    if error is not None:
+        # current_integrator_block = re.match
+        error_text = f', ERROR = {error}'
+
+        acf_text = ACF_INTEGRATOR_ERROR_PATTERN.sub(r'\1\2' + error_text + r'\4', acf_text)
+
+    # Write a temporary acf file
+    with open(acf_file + '._tmp_', 'w') as fid:
+        fid.write(acf_text)
+    
+    # Replace the existing acf file with the temporary
+    os.remove(acf_file)
+    os.rename(acf_file + '._tmp_', acf_file)   
+
+def get_motor_curve(stall_torque, max_torque, max_rpm, min_rpm, save_filename=None):    
+    """Returns data to generate a theoretical motor curve.
+    
+    Parameters
+    ----------
+    stall_torque : float
+        Torque at which the motor stalls (klbf-ft)
+    max_torque : float
+        Maximum operating torque (klbf-ft)
+    max_rpm : float
+        Maximum operating motor speed
+    min_rpm : float
+        Minimum operating motor speed
+    save_filename : bool, optional
+        If True, motor curve will be saved to file `filename`, by default None
+    
+    Returns
+    -------
+    list
+        Rpm points in normal operating range
+    list
+        Rpm points in extended stall range range
+    list
+        Torque points in normal operating range
+    list
+        Torque points in extended stall range range
+
+    Example
+    -------
+    This example plots the motor curve with a dashed line for the extended stall range.
+    >>> import matplotlib.pyplot as plt
+    >>> stall_torque = 20
+    >>> max_torque = 15
+    >>> max_rpm = 100
+    >>> min_rpm = 80
+    >>> nor_rpm, esr_rpm, nor_torque, esr_torque = get_motor_curve(stall_torque, max_torque, max_rpm, min_rpm)
+    >>> plt.plot([rpm + min_rpm for rpm in nor_rpm], nor_torque)
+    >>> plt.plot(esr_rpm, esr_torque, linestyle='--')        
+    >>> plt.show()
+
+    """
+    free_rpm = max_rpm - min_rpm
+    
+    a_nor = -max_torque/(free_rpm**2)
+    b_nor = 0
+    c_nor = max_torque
+    
+    a_esr = (stall_torque - max_torque)/(min_rpm**2)
+    b_esr = 2*(max_torque - stall_torque)/min_rpm
+    c_esr = stall_torque    
+    
+    w_nor = np.arange(0,free_rpm+1,1)
+    w_esr = np.arange(0,min_rpm,1)
+    
+    q_nor = a_nor*w_nor**2 + b_nor*w_nor + c_nor
+    q_esr = a_esr*w_esr**2 + b_esr*w_esr + c_esr
+        
+    if save_filename is not None:        
+        with open(save_filename,'w') as fid:
+            fid.write('RPM, Torque (klbf-ft)\n')
+            
+            for rpm, torque in zip(w_esr, q_esr):
+                # For each rpm and torque datapoin in the extended stall range, write the data point
+                fid.write('{},{}\n'.format(rpm, torque))
+
+            for rpm, torque in zip(w_nor, q_nor):
+                # For each rpm and torque datapoin in the normal operating range, write the data point and shift the rpm value by the rpm at max torque
+                fid.write('{},{}\n'.format(rpm+min_rpm, torque))   
+
+    return list(w_nor), list(w_esr), list(q_nor), list(q_esr)    
+    
+def get_motor_coefficients(min_rpms, max_rpms, flows):
+    """Returns the motor coefficients used in the Adams Drill motor model.
+    
+    Parameters
+    ----------
+    min_rpms : list
+        List of the minimum operating motor speeds for each flow rate.
+    max_rpms : list
+        List of the maximum operating motor speeds for each flow rate.
+    flows : list
+        List of corresponding flow rates
+    
+    Returns
+    -------
+    float
+        NOR Max (No Load) RPM A
+    float
+        NOR Max (No Load) RPM B
+    float
+        NOR Min (Max dP) RPM A
+    float
+        NOR Min (Max dP) RPM A
+
+    """
+    [nor_min_rpm_a, nor_min_rpm_b] = np.polyfit(flows, min_rpms, 1)
+    [nor_max_rpm_a, nor_max_rpm_b] = np.polyfit(flows, max_rpms, 1)
+
+    return nor_max_rpm_a, nor_max_rpm_b, nor_min_rpm_a, nor_min_rpm_b
+
+class MotorModel():
+    def __init__(self, min_rpms, max_rpms, flows, stall_torque, max_torque):
+        """Initializes an Adams Motor Model
+        
+        Parameters
+        ----------
+        min_rpms : list
+            List of the minimum operating motor speeds for each flow rate.
+        max_rpms : list
+            List of the maximum operating motor speeds for each flow rate.
+        flows : list
+            List of corresponding flow rates
+        stall_torque : float
+            Torque at which the motor stalls (klbf-ft)
+        max_torque : float
+            Maximum operating torque (klbf-ft)
+
+        """
+        nor_max_rpm_a, nor_max_rpm_b, nor_min_rpm_a, nor_min_rpm_b = get_motor_coefficients(min_rpms, max_rpms, flows)
+        self.nor_max_rpm_a = nor_max_rpm_a
+        self.nor_max_rpm_b = nor_max_rpm_b
+        self.nor_min_rpm_a = nor_min_rpm_a
+        self.nor_min_rpm_b = nor_min_rpm_b
+        self.max_torque = max_torque
+        self.stall_torque = stall_torque
+    
+    def degrade_motor(self, degradation):
+        """Degrades the motor by `degradation` (in place)
+        
+        Parameters
+        ----------
+        degradation : float
+            Motor degradation factor
+            
+        """
+        self.nor_max_rpm_a *= degradation
+        self.nor_max_rpm_b *= degradation
+        self.nor_min_rpm_a *= degradation
+        self.nor_min_rpm_b *= degradation
+        self.max_torque *= degradation
+        self.stall_torque *= degradation
+
+    def get_torque(self, rpm, flow, degradation=1):
+        """Gets the motor output torque given a motor rpm and flow rate.
+        
+        Parameters
+        ----------
+        rpm : float
+            Motor rpm
+        flow : float
+            Motor flow rate
+        degradation : float
+            Motor degradation factor (default is 1 which means there is no degradation)
+
+        Returns
+        -------
+        float
+            Motor output torque
+        
+        Raises
+        ------
+        MotorModelError
+            Raised if the rpm given is greater than the motors maximum output rpm.
+
+        """
+        nor_min_rpm_a = self.nor_min_rpm_a*degradation
+        nor_min_rpm_b = self.nor_min_rpm_b*degradation
+        nor_max_rpm_a = self.nor_max_rpm_a*degradation
+        nor_max_rpm_b = self.nor_max_rpm_b*degradation
+        max_torque = self.max_torque*degradation
+        stall_torque = self.stall_torque*degradation
+        
+
+        min_rpm = np.polyval([nor_min_rpm_a, nor_min_rpm_b], flow)*degradation
+        max_rpm = np.polyval([nor_max_rpm_a, nor_max_rpm_b], flow)*degradation
+
+        free_rpm = max_rpm - min_rpm
+        
+        if rpm < min_rpm:
+            a_coef = -max_torque/(free_rpm**2)
+            b_coef = 0
+            c_coef = max_torque
+        
+        elif rpm > max_rpm:
+            raise MotorModelError(f'The rpm provided is greater than the max rpm of {max_rpm}')
+
+        else:
+            a_coef = (stall_torque - max_torque)/(min_rpm**2)
+            b_coef = 2*(max_torque - stall_torque)/min_rpm
+            c_coef = stall_torque            
+        
+        return a_coef*rpm**2 + b_coef*rpm + c_coef
+    
+    def get_motor_curve(self, flow):
+        """Returns data t`o generate a theoretical motor curve.
+        
+        Parameters
+        ----------
+        flow : float
+            Flow rate at which to generate the motor curve
+        
+        Returns
+        -------
+        list
+            Rpm points in normal operating range
+        list
+            Rpm points in extended stall range range
+        list
+            Torque points in normal operating range
+        list
+            Torque points in extended stall range range
+
+        Example
+        -------
+        This example plots the motor curve with a dashed line for the extended stall range.
+        >>> import matplotlib.pyplot as plt
+        >>> stall_torque = 20
+        >>> max_torque = 15
+        >>> max_rpms = [80, 100, 120]
+        >>> min_rpm = [50, 70, 90]
+        >>> flows = [400, 500, 600]
+        >>> motor_model = MotorModel(min_rpms, max_rpms, flows, stall_torque, max_torque)
+        >>> nor_rpm, esr_rpm, nor_torque, esr_torque = motor_model.get_motor_curve(550)
+        >>> plt.plot([rpm + min_rpm for rpm in nor_rpm], nor_torque)
+        >>> plt.plot(esr_rpm, esr_torque, linestyle='--')        
+        >>> plt.show()
+
+        """   
+        min_rpm = np.polyval([self.nor_min_rpm_a, self.nor_min_rpm_b], flow)
+        max_rpm = np.polyval([self.nor_max_rpm_a, self.nor_max_rpm_b], flow)
+
+        nor_rpm, esr_rpm, nor_torque, esr_torque =  get_motor_curve(self.stall_torque, self.max_torque, max_rpm, min_rpm)
+
+        return min_rpm, max_rpm, nor_rpm, esr_rpm, nor_torque, esr_torque
+
+    @classmethod
+    def build_from_coeffs(cls, nor_max_rpm_a, nor_max_rpm_b, nor_min_rpm_a, nor_min_rpm_b, max_torque, stall_torque):
+        """Used to build a motor model if you already know the motor coefficients.
+        
+        Parameters
+        ----------
+        nor_max_rpm_a : float
+            nor_max_rpm_a
+        nor_max_rpm_b : float
+            nor_max_rpm_b
+        nor_min_rpm_a : float
+            nor_min_rpm_a
+        nor_min_rpm_b : float
+            nor_min_rpm_b
+        max_torque : float
+            max_torque
+        stall_torque : float
+            stall_torque
+        
+        Returns
+        -------
+        MotorModel
+            A :obj:`MotorModel` object
+
+        """
+        motor_model = cls([0, 1], [0, 1], [1, 2], stall_torque, max_torque)
+        motor_model.nor_max_rpm_a = nor_max_rpm_a
+        motor_model.nor_max_rpm_b = nor_max_rpm_b
+        motor_model.nor_min_rpm_a = nor_min_rpm_a
+        motor_model.nor_min_rpm_b = nor_min_rpm_b
+        return motor_model
+
+class MotorModelError(Exception):
+    pass
+
 class TiemOrbitSyntaxError(Exception):
     pass
 
